@@ -50,10 +50,10 @@
 
 bool full_gc = false;
 
-ref_t pre_gc_nil;
+static ref_t pre_gc_nil;
 
-unsigned long transport_count;
-unsigned long loc_transport_count;
+static unsigned long transport_count;
+static unsigned long loc_transport_count;
 
 #ifndef GC_EXAMINE_BUFFER_SIZE
 #define GC_EXAMINE_BUFFER_SIZE 16
@@ -129,10 +129,10 @@ printref(FILE * fd, ref_t refin)
       else
 	i = (long)p >> 2;
 
-      fprintf(fd, "[%ld;tag:%d;%c]", i, refin & TAG_MASK, suffix);
+      fprintf(fd, "[%ld;tag:%lu;%c]", i, (unsigned long)(refin & TAG_MASK), suffix);
     }
   else
-    fprintf(fd, "[%ld;tag:%d]", (long)(refin >> 2), refin & TAG_MASK);
+    fprintf(fd, "[%ld;tag:%lu]", (long)(refin >> 2), (unsigned long)(refin & TAG_MASK));
 }
 
 #define GC_NULL(r) ((r)==pre_gc_nil || (r)==e_nil)
@@ -401,8 +401,8 @@ GC_CHECK1(ref_t x, char *st, long i)
 #endif
 
 
-static u_int16_t *
-pc_touch(u_int16_t * o_pc)
+static instr_t *
+pc_touch(instr_t * o_pc)
 {
   ref_t *pcell = (ref_t *) ((unsigned long)o_pc & ~TAG_MASKL);
 
@@ -418,8 +418,8 @@ pc_touch(u_int16_t * o_pc)
   /* pcell++; */
 
   return
-    (u_int16_t *) ((u_int32_t) pcell
-		   | ((u_int32_t) o_pc & TAG_MASK));
+    (instr_t *) ((ref_t) pcell
+		   | ((ref_t) o_pc & TAG_MASK));
 }
 
 static void
@@ -832,7 +832,7 @@ gc_top:
    into the corresponding location in new_start.  Typically new_start
    will be the same as start */
 
-void
+static void
 shift_targets(ref_t * start, size_t len,
 	      ref_t * old_start, size_t old_len,
 	      ref_t * new_start)
