@@ -18,7 +18,7 @@
 #ifndef _INSTR_H_INCLUDED
 #define _INSTR_H_INCLUDED
 #ifndef FAST
-extern void print_instr(int /* op_field */, int /* arg_field */, u_int16_t * /* e_pc */);
-extern void print_pc(u_int16_t * /* e_pc */);
+extern void print_instr(int /* op_field */, int /* arg_field */, instr_t * /* e_pc */);
+extern void print_pc(instr_t * /* e_pc */);
 #endif
 #endif
